# -*- coding: utf-8 -*-
import json
import logging
from collections import defaultdict
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Dict

import numpy as np
from arkindex_export import Element, open_database
from line_image_extractor.extractor import extract, read_img, save_img
from line_image_extractor.image_utils import polygon_to_bbox, resize
from PIL import Image
from tqdm import tqdm

from atr_data_generator.arguments import CommonArgs
from atr_data_generator.extract.arguments import FilterArgs, ImageArgs, SelectArgs
from atr_data_generator.extract.db import get_children, get_children_info
from atr_data_generator.extract.utils import _is_vertical, resize_image_height
from atr_data_generator.split.arguments import Partition, SplitArgs
from atr_data_generator.utils import download_image, export_parameters

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

IIIF_URL_SUFFIX = "/full/full/0/default.jpg"
EXPORT_PATH = "labels.json"


@dataclass(init=False)
class DataGenerator:
    common: CommonArgs
    image: ImageArgs
    filter: FilterArgs
    select: SelectArgs
    split: SplitArgs

    data: Dict[str, dict] = field(default_factory=lambda: defaultdict(dict))

    def __init__(
        self,
        common: CommonArgs,
        image: ImageArgs,
        filter: FilterArgs,
        select: SelectArgs,
        split: SplitArgs,
        **kwargs,
    ) -> None:
        self.common = common
        self.image = image
        self.filter = filter
        self.select = select
        self.split = split
        self.data: Dict[str, dict] = defaultdict(dict)

    def __post_init__(self):
        # Create cache dir if non existent
        self.common.cache_dir.mkdir(exist_ok=True, parents=True)

    def connect_db(self, db_path: Path) -> None:
        open_database(str(db_path))

    def find_image_in_cache(self, image_id: str) -> Path:
        """Images are cached under a certain name. If it exists, the image is not downloaded.

        :param image_id: ID of the image. The image is saved under this name.
        :return: The path of the image if it exists in the cache.
        """
        return self.common.cache_dir / f"{image_id}.jpg"

    def retrieve_image(self, child: Element) -> np.ndarray[Any, Any]:
        """Get or download image of the element. Checks in cache before downloading.

        :param child: Processed element.
        :return: The element's image.
        """
        cached_img_path = self.find_image_in_cache(child.image.id)
        if not cached_img_path.exists():
            child_full_image = download_image(child.image.url + IIIF_URL_SUFFIX)

            # Convert if needed
            if self.image.grayscale:
                child_full_image = child_full_image.convert("L")

            # Save in cache
            child_full_image.save(cached_img_path, format="jpeg")

        return read_img(cached_img_path, self.image.grayscale)

    def parse_transcription(self, transcription: str, **kwargs):
        return transcription

    def get_image(self, child: Element, destination: Path) -> None:
        """Save the element's image to the given path and applies any image operations needed.

        :param child: Processed element.
        :param destination: Where the image should be saved.
        """
        polygon = json.loads(str(child.polygon))

        # Rescale if needed
        if self.image.scale.should_resize_polygons:
            polygon = resize(
                polygon,
                child.image.width,
                child.image.height,
                self.image.scale.x,
                self.image.scale.y_top,
                self.image.scale.y_bottom,
            )
        bbox = polygon_to_bbox(polygon)
        # Skip if line is vertical and these are forbidden
        if self.filter.skip_vertical_lines and _is_vertical(bbox):
            return

        # Extract the polygon in the image
        image = extract(
            img=self.retrieve_image(child),
            polygon=np.array(polygon),
            bbox=bbox,
            extraction_mode=self.image.extraction_mode.mode,
            max_deskew_angle=self.image.max_deskew_angle,
            skew_angle=self.image.skew_angle,
            grayscale=self.image.grayscale,
        )

        # Resize to required height if needed
        if self.image.fixed_height and bbox.height != self.image.fixed_height:
            image = np.asarray(
                resize_image_height(
                    img=Image.fromarray(image),
                    fixed_height=self.image.fixed_height,
                )
            )
        # Save the image to disk
        save_img(path=destination, img=image)

    def parse_image_path(self, image_path: Path):
        return str(image_path)

    def process_parent(self, parent: Element, partition: Partition):
        """
        Process every children under this parent element.
        """
        with tqdm(
            get_children_info(
                str(parent.id),
                type=self.select.element_type,
                sources=self.filter.accepted_worker_version_ids,
            ),
            desc=f"Extracting data from {parent.type} {parent.name} for split ({partition.value})",
        ) as pbar:
            for child in pbar:
                image_path = (
                    self.common.output_dir
                    / "images"
                    / f"{parent.id}_{child.element.name.split('_')[-1]}_{child.element_id}.jpg"
                )
                # Store transcription
                self.data[partition.value][
                    self.parse_image_path(image_path)
                ] = self.parse_transcription(child.text, partition=partition)

                # Extract the image
                self.get_image(child.element, image_path)

                # Progress bar updates
                pbar.update()
                pbar.refresh()

    def export(self):
        """
        Export data to disk.
        """
        (self.common.output_dir / EXPORT_PATH).write_text(
            json.dumps(self.data, indent=2)
        )

    def run(self, db_path: Path):
        """
        Extract data from folders of elements with selected type
        """
        # Either provide folders or an existing split
        assert (
            self.split.train_folder
            or self.split.validation_folder
            or self.split.test_folder
        ), "Please provide at least one folder."

        self.connect_db(db_path)

        # Iterate over folders
        for partition, folder_id in zip(Partition, self.split.folders):
            if folder_id is None:
                continue
            # Find the parent elements
            for parent_element in get_children(folder_id, type=self.select.parent_type):
                self.process_parent(parent_element, partition)

        self.export()

        export_parameters(
            common=self.common,
            image=self.image,
            select=self.select,
            filter=self.filter,
            split=self.split,
        )
