# -*- coding: utf-8 -*-
from typing import List, Optional
from uuid import UUID

from arkindex_export import Element, Transcription
from arkindex_export.queries import list_children

from atr_data_generator.extract.arguments import MANUAL


def get_children(parent_id: UUID, type: Optional[str]):
    """Recursively list children elements.

    :param parent_id: ID of the parent element.
    :param type: Optionally filter by element type.
    :return: The filtered list of children.
    """
    query = list_children(parent_id)
    if type:
        query = query.where(Element.type == type)
    return query.order_by(Element.name)


def parse_sources(sources: List[str]):
    """List of transcriptions sources. Manual source has a different treatment.

    :param sources: List of str or MANUAL.
    :return: A peewee filter by Transcription.worker_version
    """
    query_filter = None

    if MANUAL in sources:
        # Manual filtering
        query_filter = Transcription.worker_version.is_null()
        sources.remove(MANUAL)

    # Filter by worker_versions
    if sources:
        if query_filter:
            query_filter |= Transcription.worker_version.in_(sources)
        else:
            query_filter = Transcription.worker_version.in_(sources)
    return query_filter


def get_children_info(
    parent_id: str,
    type: Optional[str],
    sources: Optional[List[str]],
):
    """Get the information about the children elements and their transcriptions. Apply all needed filters.

    :param parent_id: ID of the parent element.
    :param type: Transcriptions of elements.
    """

    elements = list_children(parent_id)

    # Filter by type
    if type:
        elements = elements.where(Element.type == type)

    # Get transcriptions
    transcriptions = Transcription.select().join(
        elements, on=(Transcription.element == elements.c.id)
    )

    # Filter by transcription source
    if sources:
        transcriptions.where(parse_sources(sources.copy()))

    # Additional ordering in case there are identical names
    return transcriptions.order_by(Transcription.element.name, Transcription.element_id)
