# -*- coding: utf-8 -*-

import logging
from dataclasses import dataclass
from pathlib import Path
from typing import Optional

from atr_data_generator.extract.base import DataGenerator
from atr_data_generator.extract.pylaia import LM_FILENAME
from atr_data_generator.extract.pylaia.arguments import PylaiaArgs
from atr_data_generator.extract.pylaia.syms import Syms
from atr_data_generator.extract.pylaia.utils import _merge
from atr_data_generator.split.arguments import Partition

logger = logging.getLogger(__name__)


@dataclass
class PylaiaDataGenerator(DataGenerator):
    pylaia: Optional[PylaiaArgs] = None
    syms: Syms = Syms()

    def __post_init__(self):
        super().__post_init__()
        if self.pylaia.syms_path:
            # Load pre existing syms for all partitions
            assert self.pylaia.syms_path.exists()
            self.syms = Syms.from_disk(self.pylaia.syms_path)
            logger.info(f"Loaded symbols from {self.pylaia.syms_path}")

    def parse_image_path(self, image_path: Path):
        """Pylaia expects the image path to be relative to their parent folder

        :param image_path: Relative path to the image
        """
        return str(image_path.relative_to(image_path.parent))

    def parse_transcription(self, transcription: str, partition: Partition, **kwargs):
        return transcription, self.syms.process_line(
            transcription.strip(), read_only=partition != Partition.Train
        )

    def export_partition(self, partition):
        """Exports 3 files
        - <partition_name>.txt, with path_to_image and tokenized transcription
        - <partition_name>_ids.txt with path_to_image only
        - text_<partition_name>.txt with path_to_image and not tokenized transcription

        The tokenized transcriptions are also exported in a separate file for LM training.
        """
        data = [(path, *value) for path, value in self.data[partition.value].items()]
        if not data:
            return

        paths, transcriptions, tokenized = zip(*data)

        # Path + tokenized
        (self.common.output_dir / partition.value).with_suffix(".txt").write_text(
            "\n".join(_merge(paths, tokenized))
        )

        # Path + not tokenized
        (self.common.output_dir / f"{partition.value}_no_space.txt").write_text(
            "\n".join(_merge(paths, transcriptions))
        )

        # Paths
        (self.common.output_dir / f"{partition.value}_ids.txt").write_text(
            "\n".join(paths)
        )

        # Export data for LM training
        if partition == Partition.Train:
            (self.common.output_dir / LM_FILENAME).write_text("\n".join(tokenized))

    def export(self):
        # Export syms of training partition
        self.syms.export(self.common.output_dir)

        # Export each partition
        for partition in Partition:
            self.export_partition(partition)
