# -*- coding: utf-8 -*-
from collections import Counter
from pathlib import Path

from atr_data_generator.extract.pylaia import (
    CTC_CHAR,
    FILENAME,
    LEXICON_NAME,
    SPACE_CHAR,
    TOKEN_NAME,
    UNK_CHAR,
)
from atr_data_generator.extract.pylaia.utils import _merge


class Syms:
    """
    Class to transform word transcriptions to char transcriptions and create a symbols file.
    """

    def __init__(self):
        self._syms = set()

        self._unk_chars = Counter()
        self._char_counter = Counter()

    @property
    def syms(self):
        return list(sorted(self._syms))

    def char_replace(self, c: str, read_only: bool):
        self._char_counter.update(c)
        if c in self._syms:
            return c
        elif c.isspace():
            return SPACE_CHAR
        elif read_only:
            self._unk_chars.update(c)
            return UNK_CHAR
        else:
            self._syms.add(c)
            return c

    def process_line(self, line, read_only: bool):
        return " ".join(
            list(map(lambda x: self.char_replace(x, read_only=read_only), line))
        )

    def export(self, output: Path):
        extended_symbols = [CTC_CHAR] + self.syms + [UNK_CHAR, SPACE_CHAR]
        (output / FILENAME).write_text(
            "\n".join((f"{s} {i}" for i, s in enumerate(extended_symbols)))
        )

        # Save the list of tokens. Similar to syms.txt, but without index
        (output / TOKEN_NAME).write_text("\n".join(extended_symbols))

        # Decompose each word/character in tokens
        #  * word-based language model: "word w o r d"
        #  * character-based language model: "c c"
        # Note: special tokens like <ctc>, <space> or <unk> should NOT be decomposed in < c t c >
        (output / LEXICON_NAME).write_text(
            "\n".join(_merge(extended_symbols, list(map(" ".join, extended_symbols))))
        )

        # Character occurrences.
        characters, occurrences = zip(*self._char_counter.most_common())
        (output / FILENAME).with_suffix(".count").write_text(
            "\n".join(
                map(lambda char, count: f"{char}: {count}", characters, occurrences)
            )
        )

    @classmethod
    def from_disk(cls, filepath: Path):
        syms = Syms()
        syms._syms = set(
            line.split()[0] for line in filepath.read_text().strip().split("\n")
        )
        syms._syms -= {UNK_CHAR, SPACE_CHAR, CTC_CHAR}
        return syms
