# -*- coding: utf-8 -*-

from dataclasses import dataclass, fields
from enum import Enum
from typing import Optional
from uuid import UUID

from atr_data_generator.arguments import BaseArgs


class Partition(Enum):
    Train = "train"
    Validation = "val"
    Test = "test"


@dataclass
class SplitArgs(BaseArgs):
    """
    Arguments related to data splitting into training, validation and test subsets.

    Args:
        train_ratio (float): Ratio of data to be used in the training set. Should be between 0 and 1.
        test_ratio (float): Ratio of data to be used in the testing set. Should be between 0 and 1.
        val_ratio (float): Ratio of data to be used in the validation set. The sum of three variables should equal 1.
        train_folder (float): ID of the training folder.
        validation_folder (float): ID of the validation folder.
        test_folder (float): ID of the testing folder.
    """

    train_ratio: float = 0.8
    test_ratio: float = 0.1
    val_ratio: float = 1 - train_ratio - test_ratio

    # Existing split from Arkindex
    train_folder: Optional[UUID] = None
    validation_folder: Optional[UUID] = None
    test_folder: Optional[UUID] = None

    @property
    def folders(self):
        """Get the folders."""
        return (self.train_folder, self.validation_folder, self.test_folder)

    def json(self):
        data = super().json()

        # String formatting for UUIDs
        for field in fields(self):
            if "folder" not in field.name:
                continue
            if (value := getattr(self, field.name)) and value is not None:
                data[field.name] = str(value)
        return data
