# -*- coding: utf-8 -*-

import logging
import random
from enum import Enum
from pathlib import Path

import numpy as np

from atr_data_generator.arguments import CommonArgs
from atr_data_generator.split.arguments import SplitArgs
from atr_data_generator.utils import export_parameters

logger = logging.getLogger(__name__)

SEED = 42
random.seed(SEED)


class Split(Enum):
    Train: str = "train"
    Test: str = "test"
    Validation: str = "val"


class PartitionSplitter:
    def __init__(
        self,
        common: CommonArgs,
        split: SplitArgs,
    ):
        self.output_dir = common.output_dir
        self.split_train_ratio = split.train_ratio
        self.split_test_ratio = split.test_ratio
        self.split_val_ratio = split.val_ratio

    def page_level_split(self, line_ids: list) -> dict:
        """
        Split pages into train, validation and test subsets.
        Don't split lines to avoid data leakage.
            line_ids (list): a list of line ids named {page_id}_{line_number}_{line_id}
        """
        # Get page ids from line ids to create splits at page level
        page_ids = ["_".join(line_id.split("_")[:-2]) for line_id in line_ids]
        # Remove duplicates and sort for reproducibility
        page_ids = sorted(set(page_ids))
        random.Random(SEED).shuffle(page_ids)
        page_count = len(page_ids)

        # Use np.split to split in three sets
        stop_train_idx = round(page_count * self.split_train_ratio)
        stop_val_idx = stop_train_idx + round(page_count * self.split_val_ratio)
        train_page_ids, val_page_ids, test_page_ids = np.split(
            page_ids, [stop_train_idx, stop_val_idx]
        )

        # Build dictionary that will be used to split lines {id: split}
        page_dict = {page_id: Split.Train.value for page_id in train_page_ids}
        page_dict.update({page_id: Split.Validation.value for page_id in val_page_ids})
        page_dict.update({page_id: Split.Test.value for page_id in test_page_ids})
        return page_dict

    def parse_data(self):
        # Get all images ids (and remove extension)
        lines_path = self.output_dir / "Lines"
        return [
            str(file.relative_to(lines_path).with_suffix(""))
            for file in sorted(lines_path.rglob("*.jpg"))
        ]

    def create_partitions(self):
        """
        Data should be in output_dir / "Lines"
        """
        logger.info(f"Creating {[split.value for split in Split]} partitions")

        line_ids = self.parse_data()

        page_dict = self.page_level_split(line_ids)
        # extend this split for lines
        datasets = {s.value: [] for s in Split}
        for line_id in line_ids:
            page_id = "_".join(line_id.split("_")[:-2])
            split_id = page_dict[page_id]
            datasets[split_id].append(line_id)

        partitions_dir = self.output_dir / "Partitions"
        partitions_dir.mkdir(exist_ok=True)
        for split, split_line_ids in datasets.items():
            if not split_line_ids:
                logger.info(f"Partition {split} is empty! Skipping...")
                continue
            file_name = f"{partitions_dir}/{Split(split).name}Lines.lst"
            Path(file_name).write_text("\n".join(split_line_ids) + "\n")
        return datasets


def main(common: CommonArgs, split: SplitArgs):
    data_partitioner = PartitionSplitter(common=common, split=split)

    # create partitions from all the extracted data
    datasets = data_partitioner.create_partitions()

    export_parameters(
        common=common,
        split=split,
        datasets=datasets,
    )
