# -*- coding: utf-8 -*-
from __future__ import annotations

import getpass
import json
import logging
import socket
from datetime import datetime
from io import BytesIO
from typing import TYPE_CHECKING, Optional

import requests
import yaml
from PIL import Image
from tenacity import (
    retry,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential,
)

if TYPE_CHECKING:
    from atr_data_generator.arguments import CommonArgs
    from atr_data_generator.extract.arguments import FilterArgs, ImageArgs, SelectArgs
    from atr_data_generator.split.arguments import SplitArgs


logger = logging.getLogger(__name__)

# See http://docs.python-requests.org/en/master/user/advanced/#timeouts
DOWNLOAD_TIMEOUT = (30, 60)


def _retry_log(retry_state, *args, **kwargs):
    logger.warning(
        f"Request to {retry_state.args[0]} failed ({repr(retry_state.outcome.exception())}), "
        f"retrying in {retry_state.idle_for} seconds"
    )


@retry(
    stop=stop_after_attempt(3),
    wait=wait_exponential(multiplier=2),
    retry=retry_if_exception_type(requests.RequestException),
    before_sleep=_retry_log,
    reraise=True,
)
def _retried_request(url):
    resp = requests.get(url, timeout=DOWNLOAD_TIMEOUT)
    resp.raise_for_status()
    return resp


def download_image(url):
    """
    Download an image and open it with Pillow
    """
    assert url.startswith("http"), "Image URL must be HTTP(S)"
    # Download the image
    # Cannot use stream=True as urllib's responses do not support the seek(int) method,
    # which is explicitly required by Image.open on file-like objects
    resp = _retried_request(url)

    # Preprocess the image and prepare it for classification
    image = Image.open(BytesIO(resp.content))
    logger.debug(
        "Downloaded image {} - size={}x{}".format(url, image.size[0], image.size[1])
    )

    return image


def export_parameters(
    common: CommonArgs,
    image: Optional[ImageArgs] = None,
    split: Optional[SplitArgs] = None,
    select: Optional[SelectArgs] = None,
    filter: Optional[FilterArgs] = None,
):
    """
    Dump a JSON log file to keep track of parameters for this dataset
    """
    config = {"common": common.json()}
    if image:
        config["image"] = image.json()
    if split:
        config["split"] = split.json()
    if select:
        config["select"] = select.json()
    if filter:
        config["filter"] = filter.json()

    if common.log_parameters:
        # Get additional info on dataset and user
        current_datetime = datetime.now().strftime("%Y-%m-%d|%H-%M-%S")

        parameters = {}
        parameters["config"] = config
        parameters["info"] = {
            "user": getpass.getuser(),
            "date": current_datetime,
            "device": socket.gethostname(),
        }

        # Export
        parameter_file = (
            common.output_dir / f"param-{common.dataset_name}-{current_datetime}.json"
        )
        parameter_file.write_text(json.dumps(parameters, indent=2, default=str))

        logger.info(f"Parameters exported in file {parameter_file}")

    config_file = common.output_dir / "config.yaml"
    config_file.write_text(yaml.dump(config))

    logger.info(f"Config exported in file {config_file}")
